/*
 *	ctype.h		Character classification and conversion
 */

#ifndef _CTYPE_H
#define _CTYPE_H

#ifndef _COMPILER_H
#include <compiler.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

extern	unsigned char	*_ctype;

#define	_CTc	0x01		/* control character */
#define	_CTd	0x02		/* numeric digit */
#define	_CTu	0x04		/* upper case */
#define	_CTl	0x08		/* lower case */
#define	_CTs	0x10		/* whitespace */
#define	_CTp	0x20		/* punctuation */
#define	_CTx	0x40		/* hexadecimal */
#define _CTb	0x80		/* blank */

__EXTERN int isalnum __PROTO ((int c));
__EXTERN int isalpha __PROTO ((int c));
#if defined(__USE_SVID) || defined(__USE_MISC)
__EXTERN int isascii __PROTO ((int c));
__EXTERN int toascii __PROTO ((int c));
#endif /* _POSIX_SOURCE */
__EXTERN int iscntrl __PROTO ((int c));
__EXTERN int isdigit __PROTO ((int c));
__EXTERN int isgraph __PROTO ((int c));
__EXTERN int isprint __PROTO ((int c));
__EXTERN int ispunct __PROTO ((int c));
__EXTERN int isspace __PROTO ((int c));
__EXTERN int isupper __PROTO ((int c));
__EXTERN int isxdigit __PROTO ((int c));
#ifdef __USE_GNU
__EXTERN int isblank __PROTO ((int c));
#endif
#ifdef __USE_SVID
__EXTERN int _toupper __PROTO ((int c));
__EXTERN int _tolower __PROTO ((int c));
#endif

#ifndef __NO_CTYPE
#define	isalnum(c)	(_ctype[(unsigned char)(c)]&(_CTu|_CTl|_CTd))
#define	isalpha(c)	(_ctype[(unsigned char)(c)]&(_CTu|_CTl))
#if defined(__USE_SVID) || defined(__USE_MISC)
#define	isascii(c)	!((c)&~0x7F)
#define	toascii(c)	((c)&0x7F)
#endif /* SVID or MISC */
#define	iscntrl(c)	(_ctype[(unsigned char)(c)]&_CTc)
#define	isdigit(c)	(_ctype[(unsigned char)(c)]&_CTd)
#define	isgraph(c)	(!(_ctype[(unsigned char)(c)]&(_CTc|_CTs)) && (_ctype[(unsigned char)(c)]))
#define	islower(c)	(_ctype[(unsigned char)(c)]&_CTl)
#define isprint(c)      (!(_ctype[(unsigned char)(c)]&_CTc) && (_ctype[(unsigned char)(c)]))
#define	ispunct(c)	(_ctype[(unsigned char)(c)]&_CTp)
#define	isspace(c)	(_ctype[(unsigned char)(c)]&_CTs)
#define	isupper(c)	(_ctype[(unsigned char)(c)]&_CTu)
#define	isxdigit(c)	(_ctype[(unsigned char)(c)]&_CTx)

#ifdef __USE_GNU
#define isblank(c)	(_ctype[(unsigned char)(c)]&_CTb)
#endif

#ifdef __USE_SVID
#define	_toupper(c)	((c)^0x20)
#define	_tolower(c)	((c)^0x20)
#endif

#ifdef __USE_MISC

#ifdef __GNUC__
/* use safe versions */

#define toint(c)    \
    ({typeof(c) _c = (c);     \
	    (_c <= '9') ? (_c - '0') : (toupper(_c) - 'A' + 10); })
#define isodigit(c) \
    ({typeof(c) _c = (c);      \
	    (_c >='0') && (_c<='7'); })
#define iscymf(c)   \
    ({typeof(c) _c = (c);      \
	    isalpha(_c) || (_c == '_'); })
#define iscym(c)    \
    ({typeof(c) _c = (c);      \
	    isalnum(_c) || (_c == '_'); })

#else /* you know what */

#define toint(c)	( (c) <= '9' ? (c) - '0' : toupper(c) - 'A' + 10)
#define isodigit(c)	( (c)>='0' && (c)<='7' )
#define iscymf(c)	(isalpha(c) || ((c) == '_') )
#define iscym(c)	(isalnum(c) || ((c) == '_') )

#endif /* __GNUC__ */
#endif /* __USE_MISC */

#endif /* no __NO_CTYPE */

__EXTERN int	toupper	__PROTO((int));
__EXTERN int 	tolower	__PROTO((int));

#ifdef __cplusplus
}
#endif

#endif /* _CTYPE_H */
