/*  stpcpy.c -- MiNTLib.
    Copyright (C) 1999 Guido Flohr <gufl0000@stud.uni-sb.de>

    This file is part of the MiNTLib project, and may only be used
    modified and distributed under the terms of the MiNTLib project
    license, COPYMINT.  By continuing to use, modify, or distribute
    this file you indicate that you have read the license and
    understand and accept it fully.
*/

#include <string.h>

#ifndef HAVE_WEAK_SYMBOLS
# define __stpncpy stpncpy
#endif

/* Copy no more than N characters of SRC to DEST, returning the address of
   the terminating null in DEST, if any, or else DEST + N (in which case
   the string is not necessarily null-terminated.
   
   This function could probably be optimized, feel free to send an enhanced
   version.  But its interface is really stupid (as strncpy), because the
   unused part of the destination buffer needs to get zeroed out (which is
   of no use for most applications).
   
   This implementation assumes that the routines memcpy, memset and strlen
   are highly optimized and thus it seems appropriate to use a brute force
   algorithm.  */
char*
__stpncpy (char* dest, const char* src, size_t n)
{
  size_t strlength = strlen (src);
  size_t length = strlength > n ? n : strlength;
    
  memcpy (dest, src, length);
  if (strlength > n)
    {
      memset (dest + length, 0, strlength - n);
      return dest + length;
    }
  
  return dest + n;
}
#ifdef HAVE_WEAK_SYMBOLS
weak_alias (__stpncpy, stpncpy)
#endif
